NodePathTester = class()
lel = 3
function NodePathTester:init(x)
    -- you can accept and set parameters here
    self.id = 1
    self.nodes = {}
    self.nodePositions = {}
    self.nodeRadius = 20
    self.nodeStroke = 3.8
    self.makingLine = false
    self.connections = {}
    self.drawMode = 1
    self.highLighted = nil
    self.shouldReset = false
    self.showTutorial = true
    self.path = {}
    
    parameter.enumerated("Draw Mode", {"Normal", "Start", "End"}, function(mode)
        if not (mouse.active and mouse.version ~= nil and mouse.version >= 1.2) then
            warning("Please read the white text")
        end
        self.drawMode = mode
        if mode > 1 and self.makingLine then
            table.remove(self.connections, #self.connections)
            self.makingLine = false
        end
    end)
    
    parameter.action("Find Path", function() local isError = false
        if mouse.active and mouse.version ~= nil and mouse.version >= 1.2 then
            if self.shouldReset then
                warning("Please go to \"Edit\" first")
                return
            end
            if(self.startNode == nil) then
                warning("There is no \"Start\" node")
                isError = true
            end
            if(self.endNode == nil) then
                warning("There is no \"End\" node")
                isError = true
            end
            if isError then
                return
            end
            print("Generating Path...")
            path = PathFind:solveNodes(self.startNode, self.endNode)
            if path then
                self.path = {}
                for k, node in ipairs(path) do
                    self.path[node] = true
                end
                self.shouldReset = true
                print("Path Found")
            else
                warning("There is no path this exist")
            end
        else
            warning("Please read the white text")
        end
    end)
    parameter.action("Edit", function()
        if mouse.active and mouse.version ~= nil and mouse.version >= 1.2 then
            output.clear()
            if self.shouldReset then 
                self.shouldReset = false
                for k, node in pairs(self.path) do
                    self.path[node] = nil
                end
            end
        else
            warning("Please read the white text")
        end
    end)
    
    parameter.action("Toggle Tutorial", function()
        if mouse.active and mouse.version ~= nil and mouse.version >= 1.2 then
            self.showTutorial = not self.showTutorial
        else
            warning("Please read the white text")
        end
    end)
    
    parameter.action("Clear Canvas", function()
        if mouse.active and mouse.version ~= nil and mouse.version >= 1.2 then
            for name, node in pairs(self.nodes) do
                self:removeNode(node)
            end
            self.shouldReset = false
            
            table.clear(self.path)
            output.clear()
        else
            warning("Please read the white text")
        end
    end)
    
    parameter.action("Clear Console", function()
        if mouse.active and mouse.version ~= nil and mouse.version >= 1.2 then
            output.clear()
        else
            warning("Please read the white text")
        end
    end)
    
    self.startNode = nil
    self.endNode = nil
    
    if not mouse.active or (mouse.active and (mouse.version == nil or mouse.version < 1.2))then
        viewer.mode = FULLSCREEN
    end
    
    mouse.disconnected = function()
        if #mouse == 0 then
            viewer.mode = FULLSCREEN
        end
     end
    mouse.connected = function() viewer.mode = STANDARD 
        output.clear()
    end
    
    
    
end


function NodePathTester:update()
    --print(objc.GCMouse.current)
    if not self.shouldReset and mouse.active and mouse.version ~= nil and mouse.version >= 1.2 then
        if mouse.left.pressed and mouse.inRegion then
            local allowCreate = true
            local selectedNode = nil
            for name, node in pairs(self.nodes) do
                distance = self.nodePositions[node.name]:distance(mouse.position)
                if distance <= self.nodeRadius + (self.nodeStroke/2) then
                    allowCreate = false
                    selectedNode = node
                    break
                end
            end
            if self.drawMode == 1 then
                if allowCreate and not self.makingLine then
                    self.nodes["Node "..self.id] = PathFind.Node("Node "..self.id)
                    self.nodePositions["Node "..self.id] = mouse.position
                    self.id = self.id + 1
                elseif self.makingLine  then
                    local firstNode = self.connections[#self.connections][1]
                    if not allowCreate and selectedNode ~= firstNode and selectedNode.nodeConnections[firstNode] == nil then
                        self.connections[#self.connections][2] = selectedNode
                        local dis = self.nodePositions[firstNode.name]:distance(self.nodePositions[selectedNode.name])
                        self.connections[#self.connections][3] = dis
                        PathFind:makeConnection(firstNode, selectedNode, dis)
                        self.makingLine = false
                    end
                elseif table.getn(self.nodes) > 1 then
                    self.makingLine = true
                    table.insert(self.connections, {selectedNode})
                end
            elseif self.drawMode == 2 then
                if not allowCreate then
                    self.startNode = selectedNode
                    if self.startNode == self.endNode then
                        self.endNode = nil
                    end
                else
                    print "Please select an active node to become the start node"
                end
            else
                if not allowCreate then
                    self.endNode = selectedNode
                    if self.startNode == self.endNode then
                        self.startNode = nil
                    end
                else
                    print "Please select an active node to become the end node"
                end
            end
        end
        if mouse.right.pressed and mouse.inRegion then
            if self.drawMode == 1 then
                if not self.makingLine then
                    for name, node in pairs(self.nodes) do
                        distance = self.nodePositions[node.name]:distance(mouse.position)
                        if distance <= self.nodeRadius + (self.nodeStroke/2) then
                            self:removeNode(node)
                        end
                    end
                else
                    table.remove(self.connections, #self.connections)
                    self.makingLine = false
                end
            else
                print("You need to set draw mode to \"Normal\" in order to erase")
            end
        end
        self.highLighted = nil
        for name, node in pairs(self.nodes) do
            distance = self.nodePositions[node.name]:distance(mouse.position)
            if distance <= self.nodeRadius + (self.nodeStroke/2) then
                self.highLighted = node
            end
        end
    end
end

function NodePathTester:removeNode(node)
    self.nodePositions[node.name] = nil
    self.nodes[node.name] = nil
    for i = #self.connections, 1, -1 do
        if(self.connections[i][1] == node or self.connections[i][2] == node) then
            table.remove(self.connections, i)
        end
    end
    node:delete()
    if self.startNode == node then
        self.startNode = nil
    elseif self.endNode == node then
        self.endNode = nil
    end
end

function NodePathTester:draw()
    if mouse.active and mouse.version ~= nil and mouse.version >= 1.2 then
        background(0)
        for i = 1, (self.makingLine and (#self.connections - 1) or #self.connections), 1 do
            style.push().stroke(255, 248, 0).strokeWidth(self.nodeStroke)
                if self.shouldReset and self.path[self.connections[i][1]] and self.path[self.connections[i][2]] then
                    goto skip
                end
                line(self.nodePositions[self.connections[i][1].name],self.nodePositions[self.connections[i][2].name])
            style.pop()
            ::skip::
        end
        
        if self.shouldReset then
            for i = 1, (self.makingLine and (#self.connections - 1) or #self.connections), 1 do
                style.push().stroke(0, 255, 206).strokeWidth(self.nodeStroke+2)
                if self.path[self.connections[i][1]] and self.path[self.connections[i][2]] then
                    line(self.nodePositions[self.connections[i][1].name],self.nodePositions[self.connections[i][2].name])
                end
                style.pop()
            end
        end
            
        for name, node in pairs(self.nodes) do
            style.push().fill(0).stroke(251, 255, 0).strokeWidth(self.nodeStroke)
            if self.highLighted == node and self.startNode == node then
                style.push().stroke(255, 0, 183).strokeWidth(self.nodeStroke+3)
            elseif self.highLighted == node and self.endNode == node then
                style.push().stroke(0, 255, 195).strokeWidth(self.nodeStroke+3)
            elseif(self.highLighted == node) then
                style.push().stroke(234, 166, 65).strokeWidth(self.nodeStroke+1)
            elseif self.startNode == node then
                style.push().stroke(255, 21, 0).strokeWidth(self.nodeStroke+2)
            elseif self.endNode == node then
                style.push().stroke(0, 255, 52).strokeWidth(self.nodeStroke+2)
            end
            if self.shouldReset and self.path[node] then
                style.push().stroke(0, 255, 206)
                if self.startNode == node or self.endNode == node then
                    style.push().stroke(0, 102, 255)
                end
                style.push().strokeWidth(style.strokeWidth()+1)
            end
            x, y = self.nodePositions[node.name]:unpack()
            ellipse(x, y, self.nodeRadius)
            style.pop()
        end
        
        if(self.makingLine) then
            style.push().stroke(0, 255, 206).strokeWidth(self.nodeStroke)
            if mouse.inRegion and not self.shouldReset then
                line(self.nodePositions[self.connections[#self.connections][1].name],mouse.position)
            end
            style.pop()
        end
        
        if self.showTutorial then
            local start = 20
            local offset = 30
            local gapSize = 30
            strList = {}
            style.push().fontSize(20).textAlign(LEFT).strokeWidth(0).stroke(233, 89, 80, 0)
            if not self.shouldReset then
                if self.drawMode == 1 then
                    strList = {"Left click to place node", "Right click to remove node or to stop holding connection", "Left click one node to another to make a connection"}
                elseif self.drawMode == 2 then
                    strList = {"Left click on a node to make it be the start node(should be red)"}
                else
                    strList = {"Left click on a node to make it be the end node(should be green)"}
                end
                style.push().fill(255, 65, 0)
            else
                strList = {"You are in render mode click \"Edit\" to continue editing"}
                style.push().fill(120, 88, 231)
            end
            endStrList = {"Click \"Clear Canvas\" to clear the canvas", "Click \"Toggle Tutorial\" to remove tutorial"}
            table.move(endStrList, 1, #endStrList, #strList + 1, strList)
            for k, str in ipairs(strList) do
                text(str, WIDTH - start, HEIGHT -start -offset)
                offset = offset + gapSize
            end
        end
    elseif mouse.active and (mouse.version == nil or mouse.version < 1.2) then
        background(255,0,0)
        style.push().fontSize(50).textAlign(CENTER|MIDDLE).strokeWidth(0).fill(255)
        text("Please download the latest version\n of \"Mouse Support\" on the Web Repo.\nThere is an important update. :)", WIDTH/2, HEIGHT/2)
        style.pop()
    else
        background(255,0,0)
        style.push().fontSize(50).textAlign(CENTER|MIDDLE).strokeWidth(0).fill(255)
            text("Please connect a mouse\nYou need it for this project ;)", WIDTH/2, HEIGHT/2)
        style.pop()
    end
end

function table.clear(list)
    for k in pairs (list) do
        list [k] = nil
    end
end

function table.getn(list)
    i = 0
    for k in pairs (list) do
        i = i + 1
    end
    return i
end
